/*
** Module   : NativeSecureConnection.java
** Abstract : Helper for connecting to a remote P2J server from native code, via JNI.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140211 Created initial version.
** 002 CA  20140226 Write any errors to STDERR instead of STDOUT, so they end up on the server's
**                  log.
*/

package com.goldencode.p2j.main;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.net.*;
import com.goldencode.p2j.security.SecurityManager;

/**
 * Helper class for connecting to a remote P2J server from native code, via JNI. Used by the
 * {@code spawn} tool when no password is specified and authentication needs to be done via P2J.
 */
public class NativeSecureConnection
{
   /**
    * Establish a secure connection to the target P2J server and retrieve the spawn command
    * registered with this request, via the specified UUID.
    * <p>
    * Authentication will be performed with the temporary credentials set in the environment. If
    * the secure connection is not possible, the received certificate can not be validated or 
    * if the authentication is not possible, it returns {@code null}.
    * 
    * @param    port
    *           The P2J server's secure port.
    * @param    host
    *           The P2J server's host.
    * @param    alias
    *           The certificate alias.
    * @param    uuid
    *           The UUID identifying this request.
    *           
    * @return   The spawn command or {@code null}.
    */
   public static String[] command(int port, String host, String alias, String uuid)
   {
      // establish a secure connection to the P2J server running on the specified port; use the
      // certificate targeted by the specified alias for authentication.
      
      Session sess = null;
      try
      {
         BootstrapConfig cfg = new BootstrapConfig();
         cfg.setServer(false);
         cfg.setConfigItem("net",      "connection",     "secure",       "true");
         cfg.setConfigItem("net",      "server",         "secure_port",  Integer.toString(port));
         cfg.setConfigItem("net",      "server",         "host",         host);
         cfg.setConfigItem("net",      "queue",          "conversation", "false");
         cfg.setConfigItem("security", "certificate",    "validate",     "true");
         cfg.setConfigItem("security", "truststore",     "filename",     "srv-certs.store");
         cfg.setConfigItem("security", "truststore",     "alias",        alias);
         cfg.setConfigItem("security", "authentication", "type",         "program");
         
         // Get user credentials from environment  
         String subject = System.getenv(TemporaryAccount.P2J_SUBJECT);
         String password = System.getenv(TemporaryAccount.P2J_PASSWORD);

         cfg.setConfigItem("access", "subject",  "id",   TemporaryAccount.fromHex(subject));
         cfg.setConfigItem("access", "password", "user", TemporaryAccount.fromHex(password));

         SecurityManager secMgr = SecurityManager.createInstance(cfg);
         SessionManager sessMgr = SessionManagerFactory.createLeafNode(cfg);
         
         // establish a secure connection with the remote P2J server
         sess = sessMgr.connectDirect(cfg, null, null);
         RemoteSpawner remote =
            (RemoteSpawner) RemoteObject.obtainNetworkInstance(RemoteSpawner.class, sess);
         
         // get the command in use for this UUID
         String[] cmd = remote.getCommand(uuid);
         
         return cmd;
      }
      catch (Throwable t)
      {
         // write the error to STDERR; this way, it will end up in the server's log (as the 
         // spawner's STDERR is the same as the server's).
         final String msg = "Problem retrieving the spawn command, using these settings: " +
                            "port [%d] host [%s] alias [%s] UUID [%s]";
         System.err.println(String.format(msg, port, host, alias, uuid));
         t.printStackTrace();
         
         // do not allow exceptions to propagate into native code.
         return null;
      }
      finally
      {
         if (sess != null)
         {
            sess.terminate();
         }
      }
   }
}
